#!/bin/bash
# $Id: config.bash,v 1.2 2006/06/01 11:15:48 pblankenbaker Exp $
#
# Functions for scripts which want to support the use of a configuration
# file ($HOME/.PKG.NAME.conf or /etc/PKG.NAME.conf)


# print_config
#
# Displays current configuration in format suitable for sourcing

print_config() {
    printf "#\n# configuration options for ${SCRIPT_NAME} script\n#\n\n"
  for e in $(echo $VARNAME_LIST | sort); do
    if [ "${e}" != "CONFIG_FILE" ]; then
      printf "${e}=%q\n" "${!e}";
    fi
  done
}

# check_config_file CMD
#
# This function checks the CMD value passed. Following rules are
# imposed:
#
#   "" (unset) or "no"
#     Nothing is done
#   "load" 
#     If file $HOME/.<xsl:value-of select="info/@package"/>.<xsl:value-of select="info/name"/>.sh exists it is sourced.
#     If not found error message and exit.
#   "auto" 
#     Similar to "load", but does nothing if default config not found.
#   "save" 
#     Settings are saved to $HOME/.<xsl:value-of select="info/@package"/>.<xsl:value-of select="info/name"/>.sh for future load.
#   "show"
#     Current settings are shown
#   FILE
#     We attempt to load config from FILE specified.

check_config_file() {
  local SCFG="/etc/<xsl:value-of select="info/@package"/>.<xsl:value-of select="info/name"/>.conf"
  local UCFG="${HOME}/.<xsl:value-of select="info/@package"/>.<xsl:value-of select="info/name"/>.conf"
  local CMD="$1"

  if [ "$CMD" = "no" -o "$CMD" = "" ]; then
    return 0;
  elif [ "$CMD" = "show" ]; then
    print_config
    exit 0
  elif [ "${CMD}" = "save" ]; then
    print_config > "${UCFG}"
    echo "Saved configuration as: ${UCFG}";
    echo "Will automatically be loaded at next configure invocation."
    exit
  elif [ "${CMD}" = "auto" ]; then
    for f in "${SCFG}" "${UCFG}"; do
      if [ -f "${f}" ]; then
        . "${f}"
      fi
    done
  elif [ "${CMD}" = "load" ]; then
    if [ -f "${UCFG}" ]; then
      for f in "${SCFG}" "${UCFG}"; do
        if [ -f "${f}" ]; then
          . "${f}"
        fi
      done
    else
      error "unable to read config: ${UCFG}"
      exit 1
    fi
  elif [ -f "${CMD}" ]; then
    for f in "${SCFG}" "${CMD}"; do
      if [ -f "${f}" ]; then
        . "${f}"
      fi
    done
  else
    error "for configuration files, use: --config no|save|load|show|FILE"
    exit 1
  fi
}
# $Id: dir.bash,v 1.3 2006/11/08 16:03:37 pblankenbaker Exp $
#
# bash script functions related to directories and file system info.
#
# Requires: verbose.bash, require.bash, error()

# Flag indicating that this file has been included

__DIR_BASH="true";

# realpath PATH
#
#   Takes funky PATH names (like: "/usr/bin/../lib/../../etc/hosts") and
#   returns the actual path on the file system ("/etc/hosts").
#
# Parameters
#   PATH - Original path name that you would like to convert.
#
# Returns
#   On success, echos real path and returns 0. On failure, echos nothing
#   an returns 1.
realpath() {
  local OPATH="${1}";

  # Fail if we can't find php
  if ! check_required_program PHP php; then
    error "Failed to locate 'php' on system.";
    return 1;
  fi
  
  local NPATH;

  if NPATH="$(${PHP} -r "printf(\"%s\", realpath('${OPATH}'));")" &&
     [ -e "${NPATH}" ]; then
    echo -n "${NPATH}";
    return 0;
  fi

  return 1;
}
 

# dir_create DIR
#
# Attempts to create directory DIR if it doesn't exist. Returns 0 if
# successful. Echos error message and return 1 on failure.

dir_create() {
  local DIR="${1}";

  # Nothing to do if already present
  if [ -d "${DIR}" ]; then
    return 0;
  fi

  # Fail if we can't find mkdir
  if ! check_required_program MKDIR mkdir; then
    error "Failed to locate 'mkdir' on system.";
    return 1;
  fi

  if ${MKDIR} -p "${DIR}"; then
    verbose_out "Created directory: ${DIR}";
    return 0;
  fi

  error "Failed to create directory: ${DIR}";
  return 1;
}


# dir_rm_empty DIR [UPTODIR]
#
# Attempts to remove DIR and all of its parent directories which are
# empty (up to UPTODIR). For example, the following removes all
# empty directories up to /var/cache:
#
#   dir_rm_empty /var/cache/www.mekwin.com/tmp/bogus /var/cache
#
# NOTE: We do not consider it an error condition if we can not remove
# the initial DIR supplied (we will return 1 in this case however).
#
# DIR - Initial directory to start removing from
#
# UPTODIR - Optional directory where you want to stop (if you omit, we
# assume "/" - all the way up baby).
#
# Returns 1 if failed to remove any directories (however this may or
# may not be an error according to your script). Returns 0 if at least
# one directory was removed.

dir_rm_empty() {
  local ODIR="${1}";    # original starting point
  local SDIR="${2:-/}"; # stopping point
  local DIR="${ODIR}";  # current location

  # Nothing to do if original directory doesn't exist
  if [ ! -d "${ODIR}" ]; then
    return 1;
  fi

  #
  # Fail if we required programs aren't found
  #

  if ! check_required_program RMDIR rmdir; then
    error "Failed to locate 'rmdir' on system.";
    return 1;
  fi

  if ! check_required_program DIRNAME dirname; then
    error "Failed to locate 'dirname' on system.";
    return 1;
  fi

  #
  # Iterate up directories and attempt to remove.
  #
  # NOTE: We stop at first failure. This means we don't handle weird
  # paths like: /var/cache/./url/./bogus
  #
  RC=1; # assume we won't be able to remove any directories
  local LDIR="/";
  while [ "${DIR}" != "${LDIR}" -a "${DIR}" != "${SDIR}" ]; do
    if [ ! -w "${DIR}" ]; then
      break;
    fi

    if [ -d "${DIR}" ] && ${RMDIR} "${DIR}" 1>/dev/null 2>&1; then
      RC=0; # we removed a directory
    else
      break;
    fi
    LDIR="${DIR}";
    DIR="$(${DIRNAME} "${DIR}")";
  done

  return ${RC};
}


# dir_nuke DIR
#
# If directory DIR exists, we attempt to remove it and ALL of its
# contents (recursive destroy). BE CAREFUL!
#
# If directory DIR does not exists, we do nothing (successful).
#
# If the DIR exists after we try to remove it, we issue an error
# message and return 1.
#
# We return 0 if directory no longer exists.

dir_nuke() {
  local DIR="${1}";

  # if nothing to do, then exit success
  if [ ! -d "${DIR}" ]; then
    return 0;
  fi

  if ! check_required_program RM rm; then
    error "Failed to locate 'rm' on system.";
    return 1;
  fi

  if ! ${RM} -fr "${DIR}" || [ -d "${DIR}" ]; then
    error "Failed to completely remove: ${DIR}"
    return 1;
  fi

  return 0;
}

# $Id: ip.bash,v 1.9 2006/12/05 01:43:59 rwhalb Exp $
#
# bash script functions related to IP addresses
#
# Requires:
#    require.bash
#
# These functions all return 0 and echo out results on success, and
# echo nothing and return 1 on failure. Here is the typical way to
# code to make use of these functions:
#
# if INTERFACE="$(get_interface_ip)" ]; then
#   echo "Interface=${INTERFACE}";
# else
#   echo "failed to determine";
# fi

# Flag indicating that this file has been included

__IP_BASH="true";

# get_default_interface
#
#   Determines the default interface for the machine by looking for
#   the row with the destination set to "0.0.0.0" in the routing
#   table. On success, returns 0 and prints out interface name and
#   updates the "DEFAULT_INTERFACE" global variable. On failure
#   returns 1.
#
#   NOTE: This function "caches" the value (if DEFAULT_INTERFACE is
#   already set, it immediately returns 0).

get_default_interface() {
  # If default already determined, exit now
  if [ "${DEFAULT_INTERFACE}" != "" ]; then
    printf "${DEFAULT_INTERFACE}";
    return 0;
  fi

  # Determine interface from route -n output
  if ! check_required_program ROUTE route; then
    return 1;
  fi

  if ! check_required_program AWK awk; then
    return 1;
  fi

  DEFAULT_INTERFACE="$(${ROUTE} -n 2>/dev/null | \
	${AWK} -- '$1 == "0.0.0.0" { print $8; exit 0; }')";

  # Check for failure
  if [ "$?" != "0" -o "${DEFAULT_INTERFACE}" = "" ]; then
    unset DEFAULT_INTERFACE;
    return 1;
  fi

  printf "%s" "${DEFAULT_INTERFACE}";
  return 0;
}


# get_interface_ip INTERFACE
#
#    Determines the IP address associated with a specific interface.
#    Returns 0 if able to determine and prints value to stdout, or 1 if
#    not (and prints nothing).

get_interface_ip() {
  if ! check_required_program IFCONFIG ifconfig; then
    return 1;
  fi

  if ! check_required_program AWK awk; then
    return 1;
  fi

  # Attempt to extract IP ADDRESS
  local IP_ADDRESS_ORIG="$(${IFCONFIG} ${1} 2>/dev/null | ${AWK} -- '$1 == "inet" { print $2; exit 0; }')";
  local RC="$?";
  local IP_ADDRESS="${IP_ADDRESS_ORIG//addr:/}"

  # Check for failure
  if [ "$RC" != "0" -o "${IP_ADDRESS}" = "${IP_ADDRESS_ORIG}" ]; then
    return 1;
  fi

  printf "%s" "${IP_ADDRESS}";
  return 0;
}


# get_interface_from_ip IP
#
#    Determines the interface associated with a specific IP address.
#    Returns 0 if able to determine prints out interface name, or 1 if
#    not (and prints nothing).

get_interface_from_ip() {
  if ! check_required_program IFCONFIG ifconfig; then
    return 1;
  fi

  if ! check_required_program AWK awk; then
    return 1;
  fi

  local IP_INTERFACE="$(${IFCONFIG} -a | \
	${AWK} -- '$2 == "Link" { link=$1; } '\
'  $2 == "addr:'"${1}"'" { if (link != 0) { interface=link; } }'\
'  END { if (interface == 0) exit 1; print interface; exit 0; }')";

  if [ "$?" != "0" ]; then
    return 1;
  fi

  printf "%s" "${IP_INTERFACE}";
  return 0;
}

#
# get_network_addr INTERFACE
#
#   Attempts to get network address for a particular interface. Returns 0
#   on success and prints out the network address. Returns 1 on failure and
#   prints nothing.
get_network_addr() {
  local INTERFACE="${1}";

  if ! check_required_program IPSC ipsc; then
    return 1;
  fi

  if ! check_required_program AWK awk; then
    return 1;
  fi

  local NETWORK_ADDR="";

  if NETWORK_ADDR="$(${IPSC} -i "${INTERFACE}" 2> /dev/null | \
       ${AWK} -- '$1 == "Network" && $2 == "address:" { print $3; }')"; then
    printf "%s" "${NETWORK_ADDR}";
    return 0;
  fi
  return 1;
}

#
# get_cidr_mask INTERFACE
#
#   Attempts to get network mask for a particular interface. Returns 0
#   on success prints out mask value (0-32). Returns 1 on failure and
#   prints nothing.
get_cidr_mask() {
  local INTERFACE="${1}";

  if ! check_required_program IPSC ipsc; then
    return 1;
  fi

  if ! check_required_program GREP grep; then
    return 1;
  fi

  if ! check_required_program CUT cut; then
    return 1;
  fi

  local CIDR_MASK="";
  if CIDR_MASK="$(${IPSC} -c -i "${INTERFACE}" 2> /dev/null | \
       ${GREP} "CIDR" | ${CUT} -d/ -f2)"; then
    printf "%s" "${CIDR_MASK// /}";
    return 0;
  fi
  return 1;
}

#
# get_full_subnet_mask INTERFACE
#
#   Attempts to get network mask for a particular interface. Returns 0
#   on success prints out the full submask value in dot notation:
#   (Ex: 255.255.255.0). Returns 1 on failure and prints nothing.
get_full_subnet_mask() {
  local INTERFACE="${1}";

  if ! check_required_program IPSC ipsc; then
    return 1;
  fi

  if ! check_required_program GREP grep; then
    return 1;
  fi

  if ! check_required_program CUT cut; then
    return 1;
  fi

  local SUBNET_MASK="";
  if SUBNET_MASK="$(${IPSC} -i "${INTERFACE}" 2> /dev/null | \
       ${GREP} 'Full subnet mask:' | ${CUT} -d: -f2)"; then
    printf "%s" "${SUBNET_MASK// /}";
    return 0;
  fi
  return 1;
}

#
# get_network_addr_cidr INTERFACE
#
#   Attempts to get the IP network address for a particular
#   interface in CIDR format. Returns 0 on success and
#   prints out the IP network address CIDR format. Returns
#   1 on failure and prints nothing.
get_network_addr_cidr() {
  local INTERFACE="${1}";
  local NET_ADDR="";
  local CIDR_MASK="";

  NET_ADDR="$(get_network_addr ${INTERFACE})";
  if [ ${?} -ne 0 ]; then
    return 1;
  fi

  CIDR_MASK="$(get_cidr_mask ${INTERFACE})";
  if [ ${?} -ne 0 ]; then
    return 1;
  fi

  printf "%s\n" "${NET_ADDR}/${CIDR_MASK}";
  return 0;
}

#
# get_network_addr_subnet INTERFACE
#
#   Attempts to get the IP network address for a particular
#   interface in subnet mask format. Returns 0 on success and
#   prints out the IP network address subnet mask format.
#   Returns 1 on failure and prints nothing.
get_network_addr_subnet() {

  local INTERFACE="${1}";
  local NET_ADDR="";
  local SUBNET_MASK="";

  NET_ADDR="$(get_network_addr ${INTERFACE})";
  if [ ${?} -ne 0 ]; then
    return 1;
  fi

  SUBNET_MASK="$(get_full_subnet_mask ${INTERFACE})";
  if [ ${?} -ne 0 ]; then
    return 1;
  fi

  printf "%s\n" "${NET_ADDR}/${SUBNET_MASK}";
  return 0;
}

#
# get_broadcast_addr INTERFACE
#
#   Attemps to get the IP broadcast address for a particular
#   network interface device. Returns 0 on success and
#   prints out the IP broadcast address. Returns
#   1 on failure and prints nothing.
get_broadcast_addr() {
  local INTERFACE="${1}";
  local BROADCAST_ADDR="";

  if ! check_required_program GAWK gawk; then
    return 1;
  fi

  if ! check_required_program GREP grep; then
    return 1;
  fi

  if ! check_required_program IP ip; then
    return 1;
  fi

  BROADCAST_ADDR="$(${IP} address show "${INTERFACE}" | \
                    ${GREP} ' inet ' | ${GREP} "brd" | \
                    ${GAWK} '{ print $4 }')";

  printf "%s\n" "${BROADCAST_ADDR}";
  return 0;
}

#
# get_host_for_ip IP
#
# Attempts to determine a host name for a IP address.
#
# IP - IP to look up name for (like 192.168.0.2)
#
# Returns 0 on success (and prints out host name to stdout), or 1 on failure.
get_host_for_ip() {

  local IP="${1}";

  if ! check_required_program PERL perl; then
    return 1;
  fi

  #
  # Use perl function: gethostbyaddr();
  # - uses host name resolver libraries (NSSWITCH)... 
  local IP_HOST="$(
    printf '$addr="'"${IP}"'";
            $AF_INET="2";
            @numbers=split(/\./,$addr);
            $ip=pack("C4",@numbers);
            ($host)=(gethostbyaddr($ip, $AF_INET))[0];
            print "$host\n";
           ' | ${PERL};)";
  #
  # print out host and all alias names if found...
  if [ \( "$?" = "0" \) -a \( "${IP_HOST}" != "" \) ]; then
    printf "%s" "${IP_HOST}";
    return 0;
  fi

  return 1;
}

#
# get_host_for_ip_alias IP
#
# Attempts to determine a host name and all alias names
# for given IP address.
#
# IP - IP to look up name and aliases for (like 192.168.0.2)
#
# Returns 0 on success and prints out the host name and all aliases
# to stdout, or 1 on failure.
#
# **Note: If the "/etc/hosts" file contains more than 1 entry for
#         a given IP Address then only the first entry found for
#         the IP Address will be used...
get_host_for_ip_alias() {

  local IP="${1}";

  if ! check_required_program PERL perl; then
    return 1;
  fi

  #
  # Use perl function: gethostbyaddr();
  # - uses host name resolver libraries (NSSWITCH)... 
  local IP_HOST_ALIAS="$(
    printf '$addr="'"${IP}"'";
            $AF_INET="2";
            @numbers=split(/\./,$addr);
            $ip=pack("C4",@numbers);
            ($name, $alias, $addrtype, $length, @addrs) = (gethostbyaddr($ip, $AF_INET));
            print "$name\n";
            print "$alias\n";
           ' | ${PERL};)";
  #
  # print out host and all alias names if found...
  if [ \( "$?" = "0" \) -a \( -n "${IP_HOST_ALIAS}" \) ]; then
    for ip in ${IP_HOST_ALIAS}; do
      printf "%s\n" "${ip}";
    done
    return 0;
  fi

  return 1;
}

#
# get_ip_for_host hostname
#
# Attempts to determine one or more IP addresses for a given host name.
# NOTE: one or more IP addresses may be returned: one per line
#
# host - hostname to look up IP address for (like www.abc.com)
#
# Returns 0 on success (and prints out one or more IP addresses to stdout),
# or 1 on failure.
get_ip_for_host() {

  local HOSTNAME="${1}";

  if ! check_required_program PERL perl; then
    return 1;
  fi

#
# Use perl function: gethostbyname();
  ipAddrArray=($(printf '$hostname="'"${HOSTNAME}"'";\n($host, $aliases, $addrtype, $length, @addrs)=(gethostbyname($hostname)); foreach (@addrs) { print "".join(".",unpack( "C4", $_ ) ). "\n"; }' | ${PERL}));
  if [ \( "$?" = "0" \) -a \( ${#ipAddrArray[@]} -gt 0 \) ]; then
    for ip in ${ipAddrArray[@]}; do
      printf "%s\n" "${ip}";
    done
    return 0;
  fi

  return 1;
}

#
# is_IP4_Addr [network address]
#
#   This function returns true (0) if the format of the network
#   address is a IPv4 style IP address: "www.xxx.yyy.zzz".
#
#   network address - IP address in numeric form: "www.xxx.yyy.zzz".

is_IP4_Addr() {
  echo ${1} | \
    /bin/grep -e '^[[:digit:]]\{1,3\}\.[[:digit:]]\{1,3\}\.[[:digit:]]\{1,3\}\.[[:digit:]]\{1,3\}$' > /dev/null 2>&1;
  return $?;
}
# $Id: log.bash,v 1.2 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to displaying messages to stdout/stderr
#
# Requires: none

# Flag indicating that this file has been included

__LOG_BASH="true";

# error TEXT
#
#   Displays error message on stderr.

error() {
  echo "****ERROR****: $*" 1>&2
}

# errorf FMT [ARGS]
#
#   Displays error message using printf on stderr.

errorf() {
  local FMT="${1}";
  shift;
  printf "****ERROR****: ${1}" $* 1>&2
}

# warning TEXT
#
#   Displays warning message on stderr.

warning() {
  echo "***WARNING***: $*" 1>&2
}

# warningf FMT [ARGS]
#
#   Displays warning message using printf on stderr.

warningf() {
  local FMT="${1}";
  shift;
  printf "****WARNING****: ${1}" $* 1>&2
}
# $Id: long_help.bash,v 1.3 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to displaying long help information

# Flag indicating that this file has been included

__LONG_HELP_BASH="true";

# show_long_help NAME SHOW [EXIT_CODE]
#
#  This function is used to automate the handling of the long help
#  associated with the script. Typical invocation is of the form:
#
#    show_long_help nstsetproxy ${LONG_HELP} 0
#
# This script only displays the long help if the SHOW parameter
# matches "true". If the SHOW parameter is "true" and a EXIT_CODE
# is specified, then this script will EXIT with the exit code
# specified.
#
# NOTE: We use the browser specified in the TEXTBROWSER environment
# variable, however, if it isn't set, we look for likely candidates.
#
# NAME - Name of the NST script you want help.
#
# SHOW - If this argument is "true", then we will try to display help
# information, otherwise we will simply return.
#
# EXIT_CODE - If you want your script to terminate after displaying
# long help, pass a integer exit code as the second parameter.
#

show_long_help() {
  local SCRIPT_NAME=${1:-MISSING}

  # See if we should display long help
  if [ "${2:-false}" = "true" ]; then
    # try and locate long help file
    local HTMLHELP="local/share/doc/${SCRIPT_NAME}/${SCRIPT_NAME}.man.html"
    for f in "${PWD}/${SCRIPT_NAME}.man.html" "/usr/local/${HTMLHELP}" "/usr/${HTMLHELP}" "/mnt/cdrom/${HTMLHELP}"; do
      if [ -f "${f}" ]; then
        HTMLHELP="${f}"
        break;
      fi
    done

    # error if unable to locate help file
    if [ ! -f "${HTMLHELP}" ]; then
      show_usage
      error "did not find long help file for ${SCRIPT_NAME}"
    else
      # try and locate browser
      if [ ! -x "${TEXTBROWSER:-djkffd78none}" ]; then
        TEXTBROWSER=none
        for f in elinks lynx links w3m; do
          TEXTBROWSER="$(which "${f}" 2>/dev/null)"
          if [ "$?" = "0" ]; then break; fi
        done
      fi

      if [ -x "${TEXTBROWSER}" ]; then
        "${TEXTBROWSER}" "file://${HTMLHELP}"
      else
        show_usage
        error "did not find text browser fell back to short help "\
"- try setting TEXTBROWSER"
      fi
    fi

    # exit if told to do so after displaying help
    if [ "${3:-no_exit}" != "no_exit" ]; then
      exit ${3};
    fi

    # indicate the help was displayed
    return 1;
  fi
}
# $Id: nstscript_init.bash,v 1.2 2005/11/21 17:54:49 pblankenbaker Exp $
#
# Initialization functions for setting up a NST script

# Flag indicating that this file has been included

__NSTSCRIPT_INIT_BASH="true";

# nstscript_init SCRIPT_NAME SCRIPT_YEAR
#
# This function should be the first thing your NST script does. It takes
# care of the standard following chores (in the order shown):
#
# - Sets the SCRIPT_NAME and SCRIPT_CREATED_YEAR environment variables.
#
# - Checks to see if short help, long help or version information should be
#   shown (displays and exits if necessary).
#
# - Verifies that any required command line arguments are present.

nstscript_init() {
  #
  # Set name of script/initial year created
  export SCRIPT_NAME="${1}";
  export SCRIPT_CREATED_YEAR="${2}";

  #
  # If any obvious errors, show the usage and exit now!
  #

  if [ "$OK" != "true" ]; then
    show_usage;
    exit 2
  fi

  #
  # Check for standard help/version args and then verify required arguments
  # are specified
  show_short_help ${SCRIPT_NAME} ${SHORT_HELP} 0;
  show_long_help ${SCRIPT_NAME} ${LONG_HELP} 0;
  show_version ${SCRIPT_NAME} ${VERSION} 0;

  #
  # Verify that any required command line arguments are present
  check_required_args 1;
  
}
# $Id: passwd.bash,v 1.5 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to passwords
#
# Requires:
#    require.bash


# Flag indicating that this file has been included

__PASSWD_BASH="true";

# prompt_passwd PROMPT
#
#    Prompts user to enter password. On success, returns 0 and sets
#    the PROMPT_EPASSWD environment variables to the value entered by
#    the user. NOTE: This is the 'raw' prompting for user input
#    without echo - typically one uses prompt_verify_passwd to get a
#    validated password from user (or verifies the user input in some manner).
#
#    Example:
#
#    if prompt_passwd "New password:" && verify_passwd "${PROMPT_PASSWD}"; then
#      echo "New password looks reasonable";
#    fi

prompt_passwd() {
  unset PROMPT_PASSWD;
  if read -s -p "$1" PROMPT_PASSWD; then
    echo;
    return 0;
  fi
  unset PROMPT_PASSWD;
  echo;
  return 1;
}

# verify_passwd PASSWD
#
#   Validates that PASSWD looks reasonable [6,31] characters long and no
#   obviously bad characters.
#
#   PASSWD - Password to be checked.
#
#   Returns 0 if OK to use password, 1 if NOT

verify_passwd() {
  local VERIFY_PASSWD="${1}";

  if ((${#VERIFY_PASSWD} < 6)); then
    error "password must be at least 6 characters long";
    return 1;
  fi

  if ((${#VERIFY_PASSWD} > 31)); then
    error "password must be less than 32 characters long";
    return 1;
  fi

  if [ "${VERIFY_PASSWD//[a-zA-Z0-9\!@#\$^&*()_=+\[\]{\}|;\':\",.?\`~<>]/}" != "" ]; then

    require_program CAT cat;
    ${CAT} <<EOF
***ERROR*** limit password characters to alphanumerics  (a-z A-Z 0-9), and:
  ! @ # \$ ^ & * ( ) _ = + [ ] { } | ; ' : " , . ? \` ~ < >

  Avoid: % - \\ /
EOF
    return 1;
  fi

  # Passed all checks - return 0
  return 0;

}

# prompt_verify_passwd PROMPT
#
#    Prompts user to enter password, then prompts them again to verify
#    that they entered the same password. On success, result is stored
#    in PROMPT_PASSWD environment variable and 0 is returned, on
#    failure 1 is returned.
#
#    Example:
#
#    if prompt_verify_passwd "Set password:"; then
#      echo "Thanks for setting a new password of ${PROMPT_PASSWD}";
#    fi

prompt_verify_passwd() {
  local PROMPT="${1:-New password:}";

  while true; do
    if ! prompt_passwd "${PROMPT}"; then
      error "failed to get password from prompt"
    elif ! verify_passwd "${PROMPT_PASSWD}"; then
      unset PROMPT_PASSWD;
      continue;
    else
      local ORIGINAL_ENTRY="${PROMPT_PASSWD}";
      if prompt_passwd "Retype new password:" &&\
	 [ "${PROMPT_PASSWD}" != "${ORIGINAL_ENTRY}" ]; then
	unset PROMPT_PASSWD;
	ORIGINAL_ENTRY="";
        error "passwords do not match!"
      else
        break;
      fi
    fi
  done

  return 0;
}

# get_md5_passwd PLAIN_TEXT
#
#   Converts PLAIN_TEXT password to its MD5 equivalent (like you would
#   find in a /etc/shadow file on a RedHat system). Returns 0 on
#   success and prints out MD5 equivalent.
#   On failure, returns 1 if could not find grub-md5-crypt, or 2 if 
#   encyrption failed.

get_md5_passwd() {
  if ! check_required_program GRUB_MD5_CRYPT grub-md5-crypt; then
    error "could not find grub-md5-crypt on system";
    return 1;
  fi

  local PASSWD_MD5="$(printf "${1}\n${1}\n" | ${GRUB_MD5_CRYPT} 2>/dev/null | grep -v word)"
  if [ $? != "0" ]; then
    return 2;
  fi

  printf "%s" "${PASSWD_MD5}";
  return 0;
}
# create_ramdisk DEF_RAMDEV DEF_RAMMB
#
#   Creates a RAM disk (if not yet created) based upon combination of:
#
#   - Command line arguments
#   - Default values passed to this script invocation
#
#   On successful completion, returns 0 and sets following variables:
#
#   RAMDISK_MNT - Directory RAM disk is mounted at
#   RAMDISK_DEV - Device RAM disk is mounted from

# Flag indicating that this file has been included

__RAMDISK_BASH="true";

create_ramdisk() {
  require_program CREATERAMDISK "create_ramdisk" "/root/bin";
  require_program BASENAME "basename";

  #
  # Set default values for things user did not specify
  #
  if [ "${ramDevice}" == "DEFAULT" ]; then
    ramDevice="${1}";
  fi
    
  if [ "${ramMntPt}" == "DEFAULT" ]; then
    ramMntPt="/mnt/$(${BASENAME} "${ramDevice}")";
  fi

  if [ "${ramDiskSize}" == "DEFAULT" ]; then
     ramDiskSize="${2}";
  fi

  #
  # Go setup (or detect existence of) RAM disk
  #
  verbose_out;
  verbose_out "*** Creating a ${ramDiskSize}MByte RAM disk at mount point: \"${ramMntPt}\"..."

  # Build command to create RAM disk
  #
  createRAMDisk="${CREATERAMDISK}";
  if [ "${VERBOSE}" == "true" ]; then
    createRAMDisk="${createRAMDisk} -v";
  fi
  createRAMDisk="${createRAMDisk} -s ${ramDiskSize} -d ${ramDevice} -m ${ramMntPt}";

  if [ "${VERBOSE}" != "true" ]; then
    createRAMDisk="${createRAMDisk} > /dev/null 2>&1"
  fi
  verbose_out "${createRAMDisk}";
  if (! eval ${createRAMDisk}); then
    error "Failed to create RAM disk";
    return 1;
  fi

  #
  # Set information about RAM disk created
  #
  RAMDISK_MNT="${ramMntPt}";
  RAMDISK_DEV="${ramDevice}";
  
  return 0;
}


# create_runtime_dir PKG DEF_RAMDEV DEF_RAMMB
#
#    Creates runtime directory for PKG based upon combination of:
#
#    - Command line arguments
#    - /etc/nst.conf
#    - State of the system
#
#    Typically this results in /dev/ram4 mounted to /mnt/ram4 (for livecd)
#    or /var/nst for hardisk install.
#
# PKG - The package name to setup (like "snort")
# DEF_RAMDEV - The default ram device (like /dev/ram4)
# DEF_RAMSIZE - The default ram size (like 64)
#
# returns 0 on success and sets RUNTIME_DIR to directory to be used
# and PKG_DIR to "${RUNTIME_DIR}/${PKG}". Returns 1 on failure.

create_runtime_dir() {
  local PKG="${1:-nst}";
  local RAMDEV="${2:-/dev/ram4}";
  local RAMMB="${3:-64}";

  #
  # Handle case where user specified the run time directory
  #
  if [ "${runtimeDir}" != "DEFAULT" ]; then
    if [ -d "${runtimeDir}" ]; then
      verbose_out "User specified run time dir of: ${runtimeDir}";
      RUNTIME_DIR="${runtimeDir}";
    else
      error "Run time directory \"${runtimeDir}\" does not exist";
      return 1;
    fi

  #
  # Handle case where user specified a RAM DISK
  #
  elif [ "${ramDevice}" != "DEFAULT" -o "${ramDiskSize}" != "DEFAULT" \
	-o "${ramMntPt}" != "DEFAULT" ]; then
    if create_ramdisk "${RAMDEV}" "${RAMMB}"; then
      RUNTIME_DIR="${RAMDISK_MNT}";
    else
      return 1;
    fi

  #
  # Handle default case (where we see if hard disk directory exists, or
  # use script default ram disk).
  #
  elif [ -r "/etc/nst.conf" ] && . "/etc/nst.conf"; then
    if [ -d "${NSTHDDATADIR}" ]; then
      RUNTIME_DIR="${NSTHDDATADIR}";
    elif create_ramdisk "${RAMDEV}" "${RAMMB}"; then
      RUNTIME_DIR="${RAMDISK_MNT}";
    else
      return 1;
    fi 
  else
    if [ -d "/var/nst" ]; then
      RUNTIME_DIR="/var/nst";
    elif create_ramdisk "${RAMDEV}" "${RAMMB}"; then
      RUNTIME_DIR="${RAMDISK_MNT}";
    else
      return 1;
    fi 
  fi

  #
  # Now, create a package specific directory below the run time directory
  #
  PKG_DIR="${RUNTIME_DIR}/${PKG}";

  if [ ! -d "${PKG_DIR}" ]; then
    require_program MKDIR mkdir;
    if ${MKDIR} "${PKG_DIR}"; then
      verbose_out "Created package directory: ${PKG_DIR}";
    else
      error "Failed to create package directory: ${PKG_DIR}";
      return 1;
    fi
  fi

  return 0;
}
# $Id: require.bash,v 1.4 2006/03/16 14:27:16 pblankenbaker Exp $
#
# bash script functions related to locating system commands

# Flag indicating that this file has been included

__REQUIRE_BASH="true";

# check_required_program ENV CMD [EXTRA_PATH] [TEST]
#
# This function attempts to locate the command CMD, by searching the
# standard system path (and optionally EXTRA_PATH). If the command is
# not found, then 1 is returned. If the CMD is found, then the
# environment variable ENV is set to the full path and 0 is returned.
#
# NOTE: If the program you are looking for is not executable by your
# effective user ID, you can pass the TEST argument "-e" to check for
# existance instead of executable. Example:
#
#   check_required_program NSTPASSWD nstpasswd "" "-e"

check_required_program() {

  local ENAME="${1}";
  local TEST="${4:--x}"
  local d;

  # if already done, skip search
  if [ -x "${!ENAME}" ]; then
    return 0;
  fi

  # Search for program
  for d in ${3//:/ } /bin /sbin /usr/bin /usr/sbin /usr/local/bin /usr/local/sbin; do
    if [ ${TEST} "${d}/${2}" ]; then
      eval "${ENAME}=\"${d}/${2}\""
      return 0;
    fi
  done

  return 1;
}

# require_program ENV CMD [EXTRA_PATH]
#
# This function attempts to locate the command CMD, by searching the
# standard system path (and optionally EXTRA_PATH). If the command is
# not found, and error message is displayed AND the script will exit
# immediately!

require_program() {
  if ! check_required_program "${1}" "${2}" "${3}"; then
    error "Failed to locate required system command: ${2}"
    exit 1
  fi

  return 0;
}
# $Id: short_help.bash,v 1.2 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to displaying short help information

# Flag indicating that this file has been included

__SHORT_HELP_BASH="true";


# show_short_help NAME SHOW [EXIT_CODE]
#
#  This function is used to automate the handling of the short help
#  associated with the script. Typical invocation is of the form:
#
#    show_short_help nstsetproxy ${SHORT_HELP} 0
#
# This script only displays the short help if the SHOW parameter
# matches "true". If the SHOW parameter is "true" and a EXIT_CODE
# is specified, then this script will EXIT with the exit code
# specified.
#
# NOTE: We use the browser specified in the TEXTBROWSER environment
# variable, however, if it isn't set, we look for likely candidates.
#
# NAME - Name of the NST script you want help.
#
# SHOW - If this argument is "true", then we will try to display help
# information, otherwise we will simply return.
#
# EXIT_CODE - If you want your script to terminate after displaying
# short help, pass a integer exit code as the second parameter.
#

show_short_help() {
  local SCRIPT_NAME=${1:-MISSING}

  #
  # Also check for short help (-h or --help)
  #

  # See if we should display short help
  if [ "${2:-false}" = "true" ]; then
    show_usage;

    # exit if told to do so after displaying help
    if [ "${3:-no_exit}" != "no_exit" ]; then
      exit ${3};
    fi

    # indicate the help was displayed
    return 1;
  fi
}
# $Id: time.bash,v 1.4 2006/08/28 11:08:26 rwhalb Exp $
#
# bash script functions related to displaying verbose output

# Flag indicating that this file has been included

__TIME_BASH="true";

#
# get_time_stamp
#
#  This function returns the current time using the following
#  "TIME_STAMP" format: "yyyy-mm-dd hh-mm-ss"...

get_time_stamp() {
  echo "$(date "+%Y-%m-%d %H:%M:%S")";
  return 0;
}

#
# prt_sec_mark
#
#  This function returns the current second mark
#  since '00:00:00 1970-01-01 UTC'...
get_second_mark() {
  echo "$(date "+%s")";
  return 0;
}

#
# calc_time_duration <sec_time_stamp1> <sec_time_stamp2>
#
#  This function returns the duration between
#  time stamp: <sec_time_stamp1> and time stamp: <sec_time_stamp2>.
#
#  Duration format: +/-0000 hh:mm:ss
#
#  sec_time_stamp1 - time stamp 1 in seconds since '00:00:00 1970-01-01 UTC'...
#  sec_time_stamp2 - time stamp 2 in seconds since '00:00:00 1970-01-01 UTC'...
calc_time_duration() {

  local secs="0";
  local mins="0";
  local hrs="0";
  local days="0";

  local TS1=${1:-"0"};
  local TS2=${2:-"0"};

  local delta=$((TS2 - TS1));

  #
  # calculate duration sign...
  local sign="+";
  if [ ${delta} -lt 0 ]; then
    delta=$((delta * -1));
    sign="-";
  fi

  #
  # calc duartion components...
  secs=$((delta%60));
  mins=$(((delta/60)%60));
  hrs=$(((delta/3600)%24));
  days=$((delta/86400));

  printf "%s%04d %02d:%02d:%02d" "${sign}" "${days}" "${hrs}" "${mins}" "${secs}";
  return 0;
}
# $Id: urlcache.bash,v 1.4 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to fetching (and cacheing) a URL.
#
# Requires: dir.bash, verbose.bash, require.bash, error()
#
# Usable:   By any script
#
# Typical Usage:
#
#   - Script invokes url_cache_fetch URL one or more times
#   - Script invokes url_cache_clean at exit (to remove any downloaded files)

# Flag indicating that this file has been included

__URLCACHE_BASH="true";

#
# Default root to store cached files under. Your script can change this PRIOR
# to invoking url_cache_fetch URL.
#
URL_CACHED_DIR="${HOME}/tmp/nst/urlcache";

#
# Location of the next/last URL that was downloaded and added to the cache
# Always updated each time url_cache_fetch is invoked.
#
URL_CACHED_FILE="";

#
# Initially URL cache is empty - set tracking variables to match.
#
URL_CACHED_URLS=();
URL_CACHED_FILES=();
URL_CACHED_CNT=0;

# ### commentURLPasswd ### This function will comment out the password field
#			   if found in the URL passed to the function...
#
#  commentURLPasswd <URL>
#
#    URL format: "https://userid:password@host/path/filename"
#
#       returns: commented out password field in URL:
#                "https://userid:########@host/path/filename"
#
commentURLPasswd() {
  local url="";			# temp vars...

  url="$(echo "${1}" | /bin/sed -e 's,://\([^:]*\):[^@]*@,://\1:########@,g')";
  echo -n "${url}";
}

# url_cache_fetch URL [CACHE_DIR] [REFRESH]
#
# This function attempts to return a file name to the contents of the
# URL specified. If the URL has not yet been downloaded, we will
# download it and place it under the CACHE_DIR.
#
# NOTE: We try to be smart about "file://PATH" URLs (like
# file:///etc/hosts).  For a file URL, we simply set URL_CACHED_DIR to
# the PATH of the file WITHOUT actually placing a copy of the file in
# the cache.
#
# URL - The URL to retrieve ("http://mekwin.com/welcome.html")
#
# CACHE_DIR - Root directory for cache (typically omitted). Default
# value is defined by URL_CACHED_DIR.
#
# REFRESH - Set to "true" if you want to refresh the file if the server
# has a newer version available.
#
# On Success: returns 0 and sets URL_CACHED_FILE to the location
# where the downloaded file can be found.
#
# On Failure: returns 1
#
# If you want to clean the cache (at script exit), invoke url_cache_clear

url_cache_fetch() {
  local URL="${1}";			# get URL
  URL_CACHED_FILE="";                   # at this point assume not found
  local REFRESH="${3}";			# will be "true" if we need to check

  # If URL is in form file://PATH, then no need to cache
  local FILE="${URL//file:\/\/\//}";
  if [ "${FILE}" != "${URL}" ]; then
    if [ -r "/${FILE}" ]; then
      verbose_out "URL (${URL}) is for local file - no need to cache."
      URL_CACHED_FILE="/${FILE}";
      return 0;
    fi
    error "Unable to read file /${FILE} specified in: ${URL}";
    return 1;
  fi

  # Determine URL_CACHED_FILE name
  if ! url_cache_filename "${URL}" "${2}"; then
    error "Failed to convert '${URL}' to cache file name."
    return 1;
  fi

  # If already in cache - then done
  if [ -r "${URL_CACHED_FILE}" -a "${REFRESH}" != "true" ]; then
    verbose_out "File in cache: ${URL_CACHED_FILE}";
    url_cache_array_update "${URL}" "${URL_CACHED_FILE}";
    return 0;
  fi

  #
  # Make sure required programs are available
  #

  if ! check_required_program WGET wget; then
    error "Failed to locate 'wget' on system.";
    URL_CACHED_FILE="";
    return 1;
  fi

  #
  # Create the directory to store the file under if not present
  #

  if ! check_required_program DIRNAME dirname; then
    error "Failed to locate 'dirname' on system.";
    URL_CACHED_FILE="";
    return 1;
  fi

  local URL_CACHED_DIR="$(${DIRNAME} "${URL_CACHED_FILE}")";
  if ! dir_create "${URL_CACHED_DIR}"; then
    URL_CACHED_FILE="";
    return 1;
  fi

  #
  # Retrieve the file
  #

  local URL_DISPLAY="$(commentURLPasswd "${URL}")";
  verbose_tout "Starting download of: ${URL_DISPLAY}";

  # Assume download will fail
  local DOWNLOAD_OK=1;

  # See if used in mode to check for updates
  local WGET_OPTIONS="--server-response";
  if [ "${REFRESH}" = "true" ]; then
    WGET_OPTIONS="--timestamping ${WGET_OPTIONS}";
  fi

  # if file doesn't yet exist, turn on the no-clobber option. This
  # will prevent us from clobbering an existing file if the server
  # returns a name other than what we passed in the URL.
#  if [ ! -e "${URL_CACHED_FILE}" ]; then
    WGET_OPTIONS="${WGET_OPTIONS} --no-clobber";
#  fi
    
  #
  # If verbose, then allow wget output to be seen
  #

  if [ "${VERBOSE}" = "true" ]; then
    verbose_out "  ${WGET} ${WGET_OPTIONS} \"${URL_DISPLAY}\""
    (cd "${URL_CACHED_DIR}" && 
	${WGET} ${WGET_OPTIONS} "${URL}")
    DOWNLOAD_OK="$?";
    if [ ! -r "${URL_CACHED_FILE}" ]; then
      DOWNLOAD_OK="1";
    fi
  else
    (cd "${URL_CACHED_DIR}" && 
	${WGET} ${WGET_OPTIONS} "${URL}" 2>/dev/null)
    DOWNLOAD_OK="$?";

    if [ ! -r "${URL_CACHED_FILE}" ]; then
      DOWNLOAD_OK="1";
    fi
  fi

  # If download was OK, update cached array and return success
  if [ "${DOWNLOAD_OK}" = "0" ]; then
    verbose_tout "Finished download of: ${URL_DISPLAY}";
    url_cache_array_update "${URL}" "${URL_CACHED_FILE}";
    return 0;
  fi

  # In case a file was created, it needs to be removed
  if [ -f "${URL_CACHED_FILE}" ]; then
    /bin/rm -f "${URL_CACHED_FILE}";
  fi

  # Failed to retrieve  
  error "Failed to download: ${URL_DISPLAY}";
  URL_CACHED_FILE="";
  return 1;
}

# url_cache_clean 
#
# Removes all files found in the URL cache (and empty parent directories).

url_cache_clean() {
  if ! check_required_program DIRNAME dirname; then
    error "Failed to locate 'dirname' on system.";
    return 1;
  fi

  if ! check_required_program RM rm; then
    error "Failed to locate 'rm' on system.";
    return 1;
  fi

  local RC=0; # assume all removed successfully
  for ((i=0; i < URL_CACHED_CNT; i++)); do
    # Try to remove the file from cache
    local CFILE="${URL_CACHED_FILES[${i}]}";
    if [ -f "${CFILE}" ]; then
      if ${RM} -f "${CFILE}"; then
        verbose_out "Removed cached file ${CFILE} for url: ${URL_CACHED_URLS[${i}]}";
      else
        error "Failed to remove cached file ${CFILE} for url:  ${URL_CACHED_URLS[${i}]}";
        RC=1;
      fi
    fi

    # Now, try to remove empty directories
    dir_rm_empty "$(${DIRNAME} "${CFILE}")";
  done

  # Update variables to indicate cache has been cleared
  URL_CACHED_CNT=0;
  URL_CACHED_URLS=();
  URL_CACHED_FILES=();

  return ${RC};
}

# url_cache_array_update URL FILE
#
#   Updates internal array of URL locations which have been downloaded
#   and added to the cache. Only adds URL to array if not found. This
#   function is used internally by url_cache_fetch (most developers
#   don't need to worry about it.
#
#   Returns 0 if arrays updated, 1 if not.

url_cache_array_update() {
  local URL="${1}";
  local FILE="${2}";

  # Only cache if FILE exists
  if [ ! -f "${FILE}" ]; then
    error "${FILE} not found - not added to URL cache array";
    return 1;
  fi

  # If already in cache list, ignore
  for ((i=0; i < URL_CACHED_CNT; i++)); do
    if [ "${URL_CACHED_URLS[${i}]}" = "${URL}" ]; then
      return 0;
    fi
  done

  # Add to end of array
  URL_CACHED_URLS[${URL_CACHED_CNT}]="${URL}";
  URL_CACHED_FILES[${URL_CACHED_CNT}]="${FILE}";
  ((URL_CACHED_CNT++));
  return 0;
}

# url_cache_filename URL [CACHE_DIR]
#
# This function generates the filename for the corresponding URL as it
# will appear once cached by url_cache_fetch. This function is
# intended primarily as support for the url_cache_fetch function. NOTE:
# This does NOT create any files or directories.
#
# Also note, this strips any leading user:password from the front of
# the URL (for example: http://me:too@mekwin.com/welcome.html ends up
# being the same as http://mekwin.com/welcome.html).
#
# URL - The URL ("http://mekwin.com/welcome.html")
#
# CACHE_DIR - Root directory for cache (typically omitted). Default
# value is defined by URL_CACHED_DIR.
#
# Sets URL_CACHED_FILE to the location where the cached version of URL
# should be located.

url_cache_filename() {
  local URL="${1}";			# get URL
  local CDIR="${2}";			# get cache directory root
  if [ "${CDIR}" = "" ]; then CDIR="${URL_CACHED_DIR}"; fi

  # Compute file name relative to top level cache via stripping off protocol (http://) 
  local CFILE="$(echo "${URL}" | /bin/sed -e 's,://\([^:]*\):[^@]*@,://,g' \
	-e 's,[a-zA-Z]*://,,g')";

  # If default document (like: http://www.mekwin.com/), append
  # default.root.document to filename
  local IDX=$((${#CFILE}-1));       
  if [ "${CFILE:${IDX}:${IDX}}" = "/" ]; then
    CFILE="${CFILE}default.root.document";
  fi

  # Currently this always
  URL_CACHED_FILE="${CDIR}/${CFILE}";
  return 0;
}
# $Id: user.bash,v 1.3 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to user information
#
# Requires:
#    require.bash

# Flag indicating that this file has been included

__USER_BASH="true";


# running_as_root
#
#   Determines effective user ID (if EFFECTIVE_UID has not been set yet)
#   and then returns 0 if the ID is 0 (root), or 1 if not.

running_as_root() {
  if [ -z "${EFFECTIVE_UID}" ]; then
    require_program ID_PROGRAM id
    EFFECTIVE_UID="$(${ID_PROGRAM} -u 2>/dev/null)";
  fi
  [ "${EFFECTIVE_UID}" = "0" ];
}

# get_user_home USER
#
#   Gets the home directory (from /etc/passwd) for the specific USER.
#   You may use either the USER login ID, or the user's integer uid.
#
#   Returns 0 on success and echos user's home directory. Returns 1
#   on failure.
#
#   Example:
#
#   if USER_HOME="$(get_user_home pkb)"; then
#     echo "pkb's home directory: ${USER_HOME}";
#   fi

get_user_home() {
  require_program AWK awk;
  local USER_HOME="$(${AWK} -F: -- '$1 == "'${1}'" { print $6; exit 0; } $3 == "'${1}'" { print $6; exit 0; }' < /etc/passwd)";
  if [ "$?" = "0" -a -d "${USER_HOME}" ]; then
    echo -n "${USER_HOME}";
    return 0;
  fi

  return 1;
}

# get_user_from_uid UID
#
#   Determines the user's login ID (like "pkb") from their integer ID.
#   Requires user is present in /etc/passwd file
#
#   Returns 0 on success echos user's login name. Returns 1
#   on failure.

get_user_from_uid() {
  require_program AWK awk;
  local USER_FROM_UID="$(${AWK} -F: -- '$3 == "'${1}'" { print $1; exit 0; }' < /etc/passwd)";
  if [ "$?" = "0" -a "${USER_FROM_UID}" != "" ]; then
    echo -n "${USER_FROM_UID}";
    return 0;
  fi

  return 1;
}
# $Id: verbose.bash,v 1.12 2006/08/29 19:28:24 rwhalb Exp $
#
# bash script functions related to displaying verbose output

# Flag indicating that this file has been included

__VERBOSE_BASH="true";

# verbose_out TEXT
# verbose_err TEXT
#
#  This function echos TEXT, but only if the VERBOSE environment variable
#  is set to "true". It returns 0 if output was shown, or 1 if not.
#  (NOTE: verbose_err sends text to standard error instead of standard out)

verbose_out() {
  if [ "${VERBOSE}" = "true" ]; then
    echo "${1}";
    return 0;
  fi

  return 1;
}

verbose_err() {
  if [ "${VERBOSE}" = "true" ]; then
    echo "${1}" 1>&2;
    return 0;
  fi

  return 1;
}


# verbose_tout TEXT
#
#  This function echos "TIME_STAMP: TEXT", but only if the VERBOSE
#  environment variable is set to "true". It returns 0 if output was
#  shown, or 1 if not.

verbose_tout() {
  if [ "${VERBOSE}" = "true" ]; then
    echo "$(date "+%Y-%m-%d %H:%M:%S"): ${1}";
    return 0;
  fi

  return 1;
}

#
# TIME_STAMP_START - used in time stamp duration calculations...
TIME_STAMP_START="0";

#
# verbose_ts_start_out [Message Header]
#
#  This function echos "### Start Time: mm-dd-yyyy hh:mm:ss", but only if the
#  VERBOSE environment variable is set to "true". It returns 0 if output was
#  shown, or 1 if not.
#
#  The variable "${TIME_STAMP_START}" will be set...
#
#  Message Header - optional text message header (Default: "### Start Time:")...

verbose_ts_start_out() {
  if [ "${VERBOSE}" = "true" ]; then

    local msghdr="${1:-"### Start Time:"}";

    #
    # time stamp: date/time...
    TIME_STAMP_OUT="$(date "+%Y-%m-%d %H:%M:%S")";
    
    #
    # time stamp: converted to seconds since 1970-01-01...
    TIME_STAMP_START="$(date --date="${TIME_STAMP_OUT}" "+%s")";

    #
    # display msg header and date/time time stamp...
    echo "${msghdr} ${TIME_STAMP_OUT}";

    return 0;
  fi

  return 1;
}

#
# verbose_ts_end_dur_out [Message Header] [Display Separator]
#
#  This function echos "##### End Time: mm-dd-yyyy hh:mm:ss  Duration: +/-DDDD hh:mm:ss",
#  but only if the VERBOSE environment variable is set to "true". It returns 0 if output
#  was shown, or 1 if not.
#
#  The variable "${TIME_STAMP_START}" will be used to calculate the duration from the
#  time is was set till this function is called. May sure you set "${TIME_STAMP_START}"
#  prior to calling this function for expected results...
#
#  Message Header - optional text message header (Default: "### End Time:")...
#
#  Display Separator - Default to display a Separator line (Default: "true")...

verbose_ts_end_dur_out() {
  if [ "${VERBOSE}" = "true" ]; then

    local msghdr="${1:-"### End Time:"}";
    local displaySeparator="${2:-"true"}";

    #
    # time stamp: date/time...
    TIME_STAMP_OUT="$(date "+%Y-%m-%d %H:%M:%S")";

    #
    # time stamp: converted to seconds since 1970-01-01...
    TIME_STAMP_END="$(date --date="${TIME_STAMP_OUT}" "+%s")";

    #
    # build time stamp end: msg header and date/time time stamp...
    TIME_STAMP_END_OUT="${msghdr} ${TIME_STAMP_OUT}";

    #
    # calc duration since last: "${TIME_STAMP_START}" time stamp...
    TIME_STAMP_DUR="$(calc_time_duration "${TIME_STAMP_START}" "${TIME_STAMP_END}")";

    #
    # display separator?
    if [ "${displaySeparator}" = "true" ]; then
      echo "-------------------------------------------------------------";
    fi

    #
    # display msg header, date/time time stamp and duration...
    echo "${TIME_STAMP_END_OUT}  Duration: ${TIME_STAMP_DUR}";

    return 0;
  fi

  return 1;
}

#
# vprintf FMT [ARG0 [ARG1 ... ] ]
#
#    Like printf, but ONLY IF user enabled verbose output

vprintf() {

  if [ "${VERBOSE}" = "true" ]; then
    FMT="${1}"
    printf -- "$@"
  fi

}

# vcat
#
#    Disables output unless VERBOSE is set to "true". Use like:
#
#    vncserver -kill :2 2>&1 | vcat

vcat() {
  require_program CAT cat;
  if [ "${VERBOSE}" = "true" ]; then
    ${CAT};
  else
    ${CAT} > /dev/null 2>&1;
  fi
}

# verror MSG
#
#    If verbosity is enable, MSG is echoed out with a leading
#    ***ERROR*** indicator. NOTE: We allow escape characters - see
#    'echo -e' for details.
#
#    verror "failed to locate disk";

verror() {
  if [ "${VERBOSE}" = "true" ]; then
    echo -e "***ERROR*** ${1}";
  fi
}

# vsuccess MSG
#
#    If verbosity is enable, MSG is echoed out with a leading
#    +++SUCCESS+++ indicator. NOTE: We allow escape characters - see
#    'echo -e' for details.
#
#    vsuccess "found install disk at\n  ${INSTALL_DISK}";

vsuccess() {
  if [ "${VERBOSE}" = "true" ]; then
    echo -e "+ SUCCESS + ${1}";
  fi
}

# vbegin MSG
#
#    If verbosity is enable, MSG is echoed out with a indicator that
#    something is about to begin. NOTE: We allow escape characters -
#    see 'echo -e' for details.
#
#    vbegin "copying files to: ${ODIR}";

vbegin() {
  if [ "${VERBOSE}" = "true" ]; then
    echo -e "+ BEGIN   + ${1}";
  fi
}

# vend MSG
#
#    Intended to go with matching vbegin. NOTE: We allow escape characters -
#    see 'echo -e' for details.
#
#    vend "copied files to: ${ODIR}";

vend() {
  if [ "${VERBOSE}" = "true" ]; then
    echo -e "+ END     + ${1}";
  fi
}

# vnote MSG
#
#    If verbosity is enable, MSG is echoed out with a indicator that
#    something has occurred which is not really a "success" but not
#    yet a real "failure". NOTE: We allow escape characters - see
#    'echo -e' for details.
#
#    vnote "did not find X will try Y";

vnote() {
  if [ "${VERBOSE}" = "true" ]; then
    echo -e "+ NOTE    + ${1}";
  fi
}
# $Id: version.bash,v 1.3 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to version information

# Flag indicating that this file has been included

__VERSION_BASH="true";

# show_version NAME SHOW [EXIT_CODE]
#
# This function is used to automate the handling of displaying the
# version of the NST script.  The version number displayed will always
# match the version of the NST distribution. Typical invocation is of
# the form:
#
#    show_version nstsetproxy ${VERSION} 0
#
# This script only displays the version if the SHOW parameter
# matches "true". If the SHOW parameter is "true" and a EXIT_CODE
# is specified, then this script will EXIT with the exit code
# specified.
#
# NAME - Name of the NST script you want version info for.
#
# SHOW - We will only show version information if this argument is
# "true" (otherwise we do nothing and return).
#
# EXIT_CODE - If you want your script to terminate after displaying
# information, pass a integer exit code as the second parameter.
#
# Returns 0 if information was displayed (and a EXIT_CODE was not
# specified). Returns 1 if no information was displayed.

show_version() {
  local SCRIPT_NAME=${1:-MISSING}

  if [ "${2:-false}" = "true" ]; then
    # See if we have full copy right year info
    local COPYRIGHTYEAR="${BUILD_YEAR:-2005}"
    if [ "${SCRIPT_CREATED_YEAR}" != "" -a \
	 "$((SCRIPT_CREATED_YEAR < BUILD_YEAR))" = "1" ]; then
      COPYRIGHTYEAR="${SCRIPT_CREATED_YEAR}-${BUILD_YEAR}"
    fi

    cat <<EOF
${SCRIPT_NAME} ${NSTVER}
Copyright (C) ${COPYRIGHTYEAR} networksecuritytoolkit.org
This is free software; see the source for copying conditions.  There is NO
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
EOF

    # exit if told to do so after information is displayed
    if [ "${3:-no_exit}" != "no_exit" ]; then
      exit ${3};
    fi

    # indicate the information was displayed
    return 0;
  fi

  # indicate that no information was displayed
  return 1
}
# $Id: vnc.bash,v 1.7 2005/11/21 17:54:49 pblankenbaker Exp $
#
# bash script functions related to VNC handling
#
# Requires:
#    require.bash
#    user.bash

# Flag indicating that this file has been included

__VNC_BASH="true";


VNC_COUNT=0;

# get_vnc_processes
#
#   Updates arrays VNC_PID, VNC_UID, and VNC_DISPLAY to match the current
#   list of VNC servers running on the system. It also sets the VNC_COUNT
#   variable to the number of running VNC servers.

get_vnc_processes() {
  require_program PS ps;
  require_program AWK awk;

  # Retrieve array of VNC users, pids, and displays
  eval $(\
	${PS} --noheaders -C Xvnc 'ouid,pid,args' |\
	${AWK} -- '{ uid=uid " " $1; pid=pid " " $2; disp=disp " " $4; }'\
'END { printf("VNC_UID=(%s ); VNC_PID=(%s ); VNC_DISP=(%s );", uid, pid, disp); }')

  # Set count of VNC servers found
  VNC_COUNT="${#VNC_PID[@]}";
}

# update_vnc_connection_info
#
#    Updates array VNC_CON_CNT (with count of current connections) and
#    creates arrays VNC_CON_${VNC_PID[i]} with list of client connections.
#    You MUST run get_vnc_processes PRIOR to invoking this function.
#    For example:
#
#    get_vnc_processes  # Get current list of VNC processes
#    update_vnc_connection_info # Set VNC_CON_CNT and VNC_CON_[PID] arrays
#
#    Returns 0 on successful update. Returns 1 if unable to determine
#    connection information for 1 or more sessions.

update_vnc_connection_info() {
  require_program GREP grep;
  require_program LSOF lsof;
  require_program SED sed;

  local RC=0;

  # Lets cache full user names (and home directories) as well
  for ((i=0; i < VNC_COUNT; i++)); do
    eval VNC_CON_${VNC_PID[i]}=\($(${LSOF} -P -n -p ${VNC_PID[i]} | \
     ${GREP} "ESTABLISHED" | ${AWK} '{print $8}' | ${SED} -e 's,.*->\(.*\),\1,')\);
    VNC_CON_CNT[i]="$(eval echo \${#VNC_CON_${VNC_PID[i]}[@]})";
    if [ ${VNC_CON_CNT[i]} -eq 0 ]; then
      eval VNC_CON_${VNC_PID[i]}=\([0]='None'\);
    fi
  done

  return ${RC};
}

# update_vnc_user_info
#
#    Updates arrays VNC_USER and VNC_USER_HOME to match VNC_UID, but
#    with full user ID filled in for VNC_USER and user's home directory
#    filled in for VNC_USER_HOME. You MUST run get_vnc_processes PRIOR
#    to invoking this function. For example:
#
#    get_vnc_processes  # Get current list of VNC processes
#    update_vnc_user_info # Set VNC_USER and VNC_USER_HOME arrays
#
#    Returns 0 on successful update. Returns 1 if unable to determine
#    user information for 1 or more users.

update_vnc_user_info() {
  local RC=0;
  local USER_FROM_UID="";
  local USER_HOME="";

  # Lets cache full user names (and home directories) as well
  for ((i=0; i < VNC_COUNT; i++)); do
    local CUR_UID="${VNC_UID[${i}]}";

    # Initialize so we can tell if we find it by search
    USER_FROM_UID="";

    # If we already computed, use computed value
    for ((j=0; j < i; j++)); do
      if [ "${CUR_UID}" = "${VNC_UID[${j}]}" ]; then
	USER_FROM_UID="${VNC_USER[${j}]}";
	USER_HOME="${VNC_USER_HOME[${j}]}";
	break;
      fi
    done

    # If not already determined, try and look up
    if [ "${USER_FROM_UID}" = "" ]; then
      if ! USER_FROM_UID="$(get_user_from_uid "${CUR_UID}")"; then
	USER_FROM_UID="UNKNOWN";  # this would be bad!
        RC=1;
      fi
      if ! USER_HOME="$(get_user_home "${CUR_UID}")"; then
	USER_HOME="/home/UNKNOWN";
        RC=1;
      fi
    fi

    VNC_USER[${i}]="${USER_FROM_UID}";
    VNC_USER_HOME[${i}]="${USER_HOME}";
  done

  return ${RC};
}

# update_vnc_listen
#
#   Run after get_vnc_processes sets VNC_LISTENING[i] to "true" if VNC
#   session is accepting remote connections.  If a VNC server isn't
#   listening, VNC_LISTENING[i] is set to "false".
#
#   NOTE: It might be possible to confuse this method if you map to
#   non-standard ports.

update_vnc_listen() {
  require_program PS ps;
  require_program GREP grep;
  require_program SED sed;

  for ((i=0; i < VNC_COUNT; i++)); do
    VNC_LISTENING[i]="false";
  done

  for p in $(${PS} --no-headers -f -C Xvnc |\
	     ${GREP} -v -- -localhost |\
	     ${SED} -e 's/^.*-rfbport \([0-9]*\).*$/\1/g'); do
    for ((i=0; i < VNC_COUNT; i++)); do
      local PORT="$((5900+${VNC_DISP[${i}]//:/}))";
      if [ "${PORT}" = "${p}" ]; then
	VNC_LISTENING[i]="true";
	break;
      fi
    done
  done

}
# $Id: x.bash,v 1.3 2006/05/19 01:44:15 pblankenbaker Exp $
#
# bash script functions related to X
#
# There are numerous helpful functions within this file. Most people will
# be interested in:
#
# get_all_x_values  - Gets A LOT of information from X config
#
# set_diagonal_size - Sets DisplaySize line given monitor diagonal
#
# get_and_print_x_values - Dumps information about X configuration as bash
#                          variable names
#
# is_x_configured - Determine if system has X configuration.
#
# Many functions will cache information. These functions typically
# accept a "FORCE" parameter which will force the information to be
# re-read if you specify "true".

# Flag indicating that this file has been included

__X_BASH="true";

# get_and_print_x_values
#
#    Mainly used for test purposes. This function retrieves all of the
#    information it can from the X configuration and reports the
#    information retrieved as standard variable settings.
#
#   FORCE - Set to "true" if you want to force the retrieval (if not
#   set to "true" we only retrieve the information if it hasn't
#   already been determined).

get_and_print_x_values() {
  get_all_x_values "${1}";

  printf "#\n# Information about X configuration\n#\n\n";

  for e in X_CONFIGURED X_CONFIGURED_ALL \
           X_CONFIG_FILE "" X_COLOR_DEPTH \
           X_WIDTH_PIXELS X_HEIGHT_PIXELS "" \
           X_WIDTH_MM X_HEIGHT_MM X_DIAGONAL_MM "" \
           X_WIDTH_IN X_HEIGHT_IN X_DIAGONAL_IN "" \
           X_DPI X_ASPECT_RATIO X_ASPECT_TYPE; do
    if [ "${e}" == "" ]; then
      printf "\n";
    else
      printf "%s=%q;\n" "${e}" "${!e}";
    fi
  done

  return ${RC};
}

# get_all_x_values [FORCE]
#
#   Retrieves and sets the following variables (if information can be determined).
#
#   X_CONFIGURED - Will be "true" or "false"
#   X_CONFIGURED_ALL - Will be "true" if ALL variables have been set, false
#                      if we were unable to determine 1 or more values.
#   X_CONFIG_FILE - Will have the full path to the X configuration file.
#
#   X_COLOR_DEPTH - Will contain the color depth
#
#   X_WIDTH_PIXELS - Will contain the width of the display in pixels
#   X_HEIGHT_PIXELS - Will contain the height of the display in pixels
#
#   X_WIDTH_MM - Width of monitor in millimeters
#   X_HEIGHT_MM - Height of monitor in millimeters
#   X_DIAGONAL_MM - Diagonal of monitor in millimeters
#
#   X_WIDTH_IN - Width of monitor in inches
#   X_HEIGHT_IN - Height of monitor in inches
#   X_DIAGONAL_IN - Diagonal of monitor in inches
#
#   X_DPI - Dots per inch
#   X_ASPECT_RATIO - Actual aspect raito (width/height)
#   X_ASPECT_TYPE - ASCII name for aspect ratio ("unknown", "4:3" or "16:9")
#
# Parameter(s):
#
#   FORCE - Set to "true" if you want to force the retrieval (if not
#   set to "true" we only retrieve the information if it hasn't
#   already been determined).

get_all_x_values() {

  X_CONFIGURED_ALL="true";

  local RC=0;
  if ! get_monitor_dimensions "${1}"; then
    X_CONFIGURED_ALL="false";
    RC=1;
  fi
  
  return ${RC};

}

# set_display_size WIDTH_MM HEIGHT_MM
#
#    This method updates the DisplaySize setting within the X configuration
#    file for the width and height specified (in millimeters)

set_display_size() {
  local Wmm="${1}";
  local Hmm="${2}";

  if [ "${Wmm}" == "" -o "${Hmm}" == "" ]; then
    return 1;
  fi

  if ! is_x_configured; then
    return 1;
  fi

  if [ ! -w "${X_CONFIG_FILE}" ]; then
    return 1;
  fi

  if ! check_required_program SED sed; then
    return 1;
  fi

  # Remove any prior DisplaySize setting and insert our new version
  ${SED} -i -e "/.*DisplaySize.*/d" \
     -e '/.*Section.*Monitor.*/a\ \ \ \ '"DisplaySize ${Wmm} ${Hmm}" \
     "${X_CONFIG_FILE}";

}

# set_diagonal_size W_PIX H_PIX LEN in|mm
#
#    This method will set the DisplaySize line in a X configuration
#    file such that.
#
#    W_PIX - Width of screen in pixels
#    H_PIX - Height of screen in pixels
#    LEN   - Length of the diagonal
#    in|mm - Whether length is in inches or millimeters

set_diagonal_size() {
  local Wpix="${1}";
  local Hpix="${2}";
  local UNITS="${4}";

  if ! check_required_program DC dc; then
    return 1;
  fi

  # Verify units
  if [ "${UNITS}" == "in" ]; then
    local Dmm="$(echo "4 k 25.4 ${3} * p" | ${DC})";
  elif [ "${UNITS}" == "mm" ]; then
    local Dmm="${3}";
  else
    return 1;
  fi

  local Hmm="$(echo "6 k ${Dmm} 2 ^ 1 ${Wpix} ${Hpix} / 2 ^
	       + / v .5 + 0 k 1 / p" | ${DC})";

  local Wmm="$(echo "6 k ${Hmm} ${Wpix} * ${Hpix} /
               .5 + 0 k 1 / p" | ${DC})";

  set_display_size "${Wmm}" "${Hmm}";

}

# is_x_configured [FORCE]
#
#    Determines if X has been configured and if so sets the variable
#    X_CONFIG_FILE to the location of the X configuration file. It
#    also sets the variable X_CONFIG_FILE to either "true" or "false"
#    depending upon whether the configuration file was found or not.
#
#   FORCE - Set to "true" if you want to force the retrieval (if not
#   set to "true" we only retrieve the information if it hasn't
#   already been determined).

is_x_configured() {

  if [ "${1}" != "true" -a "${X_CONFIGURED}" != "" ]; then
    if [ "${X_CONFIGURED}" == "true" ]; then
      return 0;
    else
      return 1;
    fi
  fi

  X_CONFIG_FILE="/etc/X11/xorg.conf";
  if [ -f "${X_CONFIG_FILE}" ]; then
    X_CONFIGURED="true";
    return 0;
  fi

  unset X_CONFIG_FILE;
  X_CONFIGURED="false";
  return 1;

}

# get_default_color_depth [FORCE]
#
#    Look for DefaultDepth in X configuration file (or use Depth from
#    last "Display" SubSection) as a guess at the default color depth.
#    If able to determine, returns success and sets X_COLOR_DEPTH variable
#    to value found.
#
#   FORCE - Set to "true" if you want to force the retrieval (if not
#   set to "true" we only retrieve the information if it hasn't
#   already been determined).

get_default_color_depth() {

  if [ "${1}" != "true" -a "${X_COLOR_DEPTH}" != "" ]; then
    return 0;
  fi

  unset X_COLOR_DEPTH;

  if ! is_x_configured "${1}"; then
    return 1;
  fi

  if ! check_required_program AWK awk; then
    return 1;
  fi

  eval $(${AWK} -- '

  BEGIN {
    inScreenSection = 0;
    depth = 0;  
  }

  $1 == "Section" && $2 == "\"Screen\"" {
    inScreenSection = 1;
  }

  $1 == "EndSection" {
    if (inScreenSection) {
      if (depth > 0) {
        printf("X_COLOR_DEPTH=%d\n", depth);
        exit(0);
      }
      exit(1);
    }
  }

  $1 == "Depth" {
    if (inScreenSection) {
      depth = $2;
    }
  }

  $1 == "DefaultDepth" {
    if (inScreenSection) {
      printf("X_COLOR_DEPTH=%d\n", $2);
      exit(0);
    }
  }

  END {
    exit(1);
  }

' ${X_CONFIG_FILE});

}


# get_default_pixel_dimensions [FORCE]
#
#    Determine the default width and height (in pixels) of the X
#    configuration with the color depth matching the value in the
#    X_COLOR_DEPTH variable (this will also determine the default
#    color depth if X_COLOR_DEPTH has not yet been set).
#
#    If this function returns success, it will set the variables
#     X_WIDTH_PIXELS and X_HEIGHT_PIXELS.
#
#   FORCE - Set to "true" if you want to force the retrieval (if not
#   set to "true" we only retrieve the information if it hasn't
#   already been determined).

get_default_pixel_dimensions() {

  if [ "${1}" != "true" -a "${X_WIDTH_PIXELS}" != "" ]; then
    return 0;
  fi

  unset X_MODE;
  unset X_WIDTH_PIXELS;
  unset X_HEIGHT_PIXELS;

  if ! get_default_color_depth "${1}"; then
    return 1;
  fi

  if ! check_required_program AWK awk; then
    return 1;
  fi

  eval $(${AWK} -- '

  BEGIN {
    useNextMode = 0;
  }

  $1 == "Depth" && $2 == "'${X_COLOR_DEPTH}'" {
    useNextMode = 1;
  }

  $1 == "Modes" {
    if (useNextMode) {
      printf("X_MODE=%s\n", $2);
      exit 0;
    }
  }

  END {
    exit(1);
  }

' ${X_CONFIG_FILE});

  # Strip off double quotes and check for WxH
  X_MODE="${X_MODE//\"/}";
  if [ "${X_MODE}" == "" ] || [ "${X_MODE//[0-9]*x[0-9]*/}" != "" ]; then
    unset X_MODE;
    return 1;
  fi

  # Get width and height
  X_WIDTH_PIXELS="${X_MODE//x[0-9]*/}";
  X_HEIGHT_PIXELS="${X_MODE//[0-9]*x/}";

  return 0;

}


# get_monitor_dimensions [FORCE]
#
#   This function attempts to retrieve monitor dimension information
#   directly from the X configuration file. If successful, it will
#   return 0 and set the following variables:
#
#   X_WIDTH_MM - Width of monitor in millimeters
#   X_HEIGHT_MM - Height of monitor in millimeters
#   X_DIAGONAL_MM - Diagonal of monitor in millimeters
#   X_WIDTH_IN - Width of monitor in inches
#   X_HEIGHT_IN - Height of monitor in inches
#   X_DIAGONAL_IN - Diagonal of monitor in inches
#   X_DPI - Dots per inch
#   X_ASPECT_RATIO - Actual aspect raito (width/height)
#   X_ASPECT_TYPE - ASCII name for aspect ratio ("unknown", "4:3" or "16:9")
#
#   FORCE - Set to "true" if you want to force the retrieval (if not
#   set to "true" we only retrieve the information if it hasn't
#   already been determined).

get_monitor_dimensions() {

  if [ "${1}" != "true" -a "${X_DIAGONAL_IN}" != "" ]; then
    return 0;
  fi

  unset X_WIDTH_MM;
  unset X_HEIGHT_MM;
  unset X_DIAGONAL_MM;
  unset X_WIDTH_IN;
  unset X_HEIGHT_IN;
  unset X_DIAGONAL_IN;

  if ! get_default_pixel_dimensions "${1}"; then
    return 1;
  fi

  if ! check_required_program AWK awk ||
     ! check_required_program DC dc; then
    return 1;
  fi

  eval $(${AWK} -- '

  BEGIN {
    useNextSize = 0;
  }

  $1 == "Section" && $2 == "\"Monitor\"" {
    useNextSize = 1;
  }

  $1 == "DisplaySize" {
    if (useNextSize) {
      printf("X_WIDTH_MM=%d\nX_HEIGHT_MM=%d\n", $2, $3);
      exit 0;
    }
  }

  END {
    exit(1);
  }

' ${X_CONFIG_FILE});

  if [ "${?}" != 0 ] || [ "${X_WIDTH_MM}" == "" ]; then
    return 1;
  fi

  X_DIAGONAL_MM="$(echo "${X_WIDTH_MM} 2 ^ ${X_HEIGHT_MM} 2 ^ + v p" | ${DC})";

  X_WIDTH_IN="$(echo "4 k ${X_WIDTH_MM} 25.4 / p" | ${DC})";
  X_HEIGHT_IN="$(echo "4 k ${X_HEIGHT_MM} 25.4 / p" | ${DC})";
  X_DIAGONAL_IN="$(echo "4 k ${X_DIAGONAL_MM} 25.4 / p" | ${DC})";

  # Compute DPI and round to integer value
  X_DPI="$(echo "6 k ${X_WIDTH_PIXELS} ${X_HEIGHT_PIXELS} +
                     ${X_WIDTH_IN} ${X_HEIGHT_IN} + / .5 + p" | ${DC})";
  X_DPI="${X_DPI//.[0-9]*/}";

  X_ASPECT_RATIO="$(echo "4 k ${X_WIDTH_MM} ${X_HEIGHT_MM} / p" | ${DC})";

  # Make a guess at aspect ratio if within 10% of 4/3 or 16/9
  X_ASPECT_TYPE="unknown";
  local ABIG="$(echo "${X_ASPECT_RATIO} 100 * 2 k p" | ${DC})";
  ABIG="${ABIG//.[0-9]*/}";

  if (( (ABIG < 146) && (ABIG > 120) )); then
    X_ASPECT_TYPE="4:3";
  elif (( (ABIG < 194) && (ABIG > 160) )); then
    X_ASPECT_TYPE="16:9";
  fi

  return 0;
}

# set_resolution WIDTH HEIGHT
#
#    Set resolution (width and height in pixels) for the xorg.conf file.

set_resolution() {
  local WIDTH="${1}";
  local HEIGHT="${2}";
  local DISP_MODE="${WIDTH}x${HEIGHT}";

  if ! check_required_program SYSCONFDISP system-config-display; then
    return 1;
  fi

  # Use system-config-display to set values and then verify by reading 
  # them back
  ${SYSCONFDISP} --set-resolution "${DISP_MODE}" && 
    get_default_pixel_dimensions true &&
    [ "${WIDTH}" == "${X_WIDTH_PIXELS}" ] &&
    [ "${HEIGHT}" == "${X_HEIGHT_PIXELS}" ];

}
#!/bin/bash
#
# $Id: nstpatch.sh.create,v 1.1 2007/03/09 11:57:38 pblankenbaker Exp $
#
# See nstpatch.xml for summary of what this patch does
#
# NOTE: When installed, this patch will have ALL of the functions
# (every .bash file from src/include/functions pasted above).

. "/etc/nst.conf";

# File to patch
SNORTD="/etc/rc.d/init.d/snortd";

# Assume patch will be OK
PATCH_OK="true";
PATCH_DIR="${1:-"${PWD}"}";
PATCH_LABEL="${SNORTD} (patch: P200704011)";

#
# Enable verbose output and output/set start time
VERBOSE="true";

#
# Programs which we will use
require_program CHKCONFIG chkconfig;
require_program FIND find;
require_program MV mv;
require_program SED sed;

#
# Start the update
vprintf "\n";
verbose_ts_start_out;

#
# Do the actual patch
#

vbegin "Installing: ${PATCH_LABEL}";

# Determine if snortd is currently enabled at boot time
vnote "Checking to see if snortd is enabled at boot time";
${CHKCONFIG} snortd;
ENABLED=${?};

# 
if [ "${ENABLED}" == "0" ]; then
  if ${CHKCONFIG} snortd off &>/dev/null; then
    vsuccess "Temporarily disabled snortd at boot time";
  else
    verror "Failed to disable snortd at boot time";
    PATCH_OK="false";
  fi
fi

# Patch the file
if ${SED} -i -e 's/\(# chkconfig: [0-9]*\).*/\1 93 83/' ${SNORTD}; then
  vsuccess "Updated: \"${SNORTD}\"";
else
  verror "Failed to update: \"${SNORTD}\"";
  PATCH_OK="false";
fi

# Re-enable snortd at boot (if had been enabled before)
if [ "${ENABLED}" == "0" ]; then
  if ${CHKCONFIG} snortd on &>/dev/null; then
    vsuccess "Re-enabled snortd at boot time";
  else
    verror "Failed to re-enable snortd at boot time";
    PATCH_OK="false";
  fi
fi

# Clean up any lingering K30snortd links
for f in $(${FIND} /etc/rc.d -name "K30snortd"); do
  ${MV} -f "${f}" "${f//K30/K83}";
done

vend "Installing: ${PATCH_LABEL}";

#
# Dump summary results of applying patches
vprintf "\n";
if [ "${PATCH_OK}" == "false" ]; then
  verror "Patch Id: \"P200704011\", Version: \"1.0.0\" failed to install."
else
  vsuccess "Patch Id: \"P200704011\", Version: \"1.0.0\" was successfully installed.";
fi

verbose_ts_end_dur_out;
